@@ -53,15 +53,12 @@ Functions
 .. autofunction:: Merge
 
 """
-from __future__ import absolute_import
-from six.moves import range
-import six
-
 import errno
 import numpy as np
 import logging
 import copy
 import warnings
+import contextlib
 import collections
 
 import MDAnalysis
@@ -71,23 +68,11 @@ import sys
 # trigger an MPI warning because importing the uuid module triggers a call to
 # os.fork(). This happens if MPI_Init() has been called prior to importing
 # MDAnalysis. The problem is actually caused by the uuid module and not by
-# MDAnalysis itself. Python 3.7 fixes the problem. However, for Python < 3.7,
-# the uuid module works perfectly fine with os.fork() disabled during import.
-# A clean solution is therefore simply to disable os.fork() prior to importing
-# the uuid module and to re-enable it afterwards.
+# MDAnalysis itself. Python 3.7 fixes the problem.
 import os
+import uuid
 
-# Windows doesn't have os.fork so can ignore
-# the issue for that platform
-if sys.version_info >= (3, 7) or os.name == 'nt':
-    import uuid
-else:
-    _os_dot_fork, os.fork = os.fork, None
-    import uuid
-    os.fork = _os_dot_fork
-    del _os_dot_fork
-
-from .. import _ANCHOR_UNIVERSES, _TOPOLOGY_ATTRS, _PARSERS
+from .. import _TOPOLOGY_ATTRS, _PARSERS
 from ..exceptions import NoDataError
 from ..lib import util
 from ..lib.log import ProgressBar
@@ -99,9 +84,10 @@ from .groups import (ComponentBase, GroupBase,
                      Atom, Residue, Segment,
                      AtomGroup, ResidueGroup, SegmentGroup)
 from .topology import Topology
-from .topologyattrs import AtomAttr, ResidueAttr, SegmentAttr
+from .topologyattrs import AtomAttr, ResidueAttr, SegmentAttr, BFACTOR_WARNING
 from .topologyobjects import TopologyObject
 
+
 logger = logging.getLogger("MDAnalysis.core.universe")
 
 
@@ -129,7 +115,7 @@ def _topology_from_file_like(topology_file, topology_format=None,
         if (err.errno is not None and
             errno.errorcode[err.errno] in ['ENOENT', 'EACCES']):
             # Runs if the error is propagated due to no permission / file not found
-            six.reraise(*sys.exc_info())
+            raise sys.exc_info()[1] from err
         else:
             # Runs when the parser fails
             raise IOError("Failed to load from the topology file {0}"
@@ -142,11 +128,8 @@ def _topology_from_file_like(topology_file, topology_format=None,
             "Error: {2}".format(topology_file, parser, err))
     return topology
 
-# py3 TODO
-#def _resolve_formats(*coordinates, format=None, topology_format=None):
-def _resolve_formats(*coordinates, **kwargs):
-    format = kwargs.get('format', None)
-    topology_format = kwargs.get('topology_format', None)
+
+def _resolve_formats(*coordinates, format=None, topology_format=None):
     if not coordinates:
         if format is None:
             format = topology_format
@@ -154,19 +137,13 @@ def _resolve_formats(*coordinates, **kwargs):
             topology_format = format
     return format, topology_format
 
-# py3 TODO
-#def _resolve_coordinates(filename, *coordinates, format=None,
-#                         all_coordinates=False):
-def _resolve_coordinates(*args, **kwargs):
-    filename = args[0]
-    coordinates = args[1:]
-    format = kwargs.get('format', None)
-    all_coordinates = kwargs.get('all_coordinates', False)
 
+def _resolve_coordinates(filename, *coordinates, format=None,
+                         all_coordinates=False):
     if all_coordinates or not coordinates and filename is not None:
         try:
             get_reader_for(filename, format=format)
-        except ValueError:
+        except (ValueError, TypeError):
             warnings.warn('No coordinate reader found for {}. Skipping '
                             'this file.'.format(filename))
         else:
@@ -200,49 +177,33 @@ def _generate_from_topology(universe):
 class Universe(object):
     """The MDAnalysis Universe contains all the information describing the system.
 
-    The system always requires a *topology* file --- in the simplest case just
+    The system always requires a *topology file* --- in the simplest case just
     a list of atoms. This can be a CHARMM/NAMD PSF file or a simple coordinate
-    file with atom informations such as XYZ, PDB, Gromacs GRO, or CHARMM
+    file with atom informations such as XYZ, PDB, GROMACS GRO or TPR, or CHARMM
     CRD. See :ref:`Supported topology formats` for what kind of topologies can
     be read.
 
-    A trajectory provides coordinates; the coordinates have to be ordered in
-    the same way as the list of atoms in the topology. A trajectory can be a
-    single frame such as a PDB, CRD, or GRO file, or it can be a MD trajectory
-    (in CHARMM/NAMD/LAMMPS DCD, Gromacs XTC/TRR, or generic XYZ format).  See
-    :ref:`Supported coordinate formats` for what can be read as a
-    "trajectory".
+    A *trajectory file* provides coordinates; the coordinates have to be
+    ordered in the same way as the list of atoms in the topology. A trajectory
+    can be a single frame such as a PDB, CRD, or GRO file, or it can be a MD
+    trajectory (in CHARMM/NAMD/LAMMPS DCD, GROMACS XTC/TRR, AMBER nc, generic
+    XYZ format, ...).  See :ref:`Supported coordinate formats` for what can be
+    read as a "trajectory".
 
     As a special case, when the topology is a file that contains atom
     information *and* coordinates (such as XYZ, PDB, GRO or CRD, see
     :ref:`Supported coordinate formats`) then the coordinates are immediately
     loaded from the "topology" file unless a trajectory is supplied.
 
-    Examples for setting up a universe::
-
-       u = Universe(topology, trajectory)          # read system from file(s)
-       u = Universe(pdbfile)                       # read atoms and coordinates from PDB or GRO
-       u = Universe(topology, [traj1, traj2, ...]) # read from a list of trajectories
-       u = Universe(topology, traj1, traj2, ...)   # read from multiple trajectories
-
-    Load new data into a universe (replaces old trajectory and does *not* append)::
-
-       u.load_new(trajectory)                      # read from a new trajectory file
-
-    Select atoms, with syntax similar to CHARMM (see
-    :class:`~Universe.select_atoms` for details)::
-
-       u.select_atoms(...)
-
     Parameters
     ----------
-    topology: str, stream, `~MDAnalysis.core.topology.Topology`, `np.ndarray`, None
+    topology: str, stream, Topology, numpy.ndarray, None
         A CHARMM/XPLOR PSF topology file, PDB file or Gromacs GRO file; used to
         define the list of atoms. If the file includes bond information,
         partial charges, atom masses, ... then these data will be available to
         MDAnalysis. Alternatively, an existing
         :class:`MDAnalysis.core.topology.Topology` instance may be given,
-        numpy coordinates, or None for an empty universe.
+        numpy coordinates, or ``None`` for an empty universe.
     coordinates: str, stream, list of str, list of stream (optional)
         Coordinates can be provided as files of
         a single frame (eg a PDB, CRD, or GRO file); a list of single
@@ -277,17 +238,6 @@ class Universe(object):
     vdwradii: dict, ``None``, default ``None``
         For use with *guess_bonds*. Supply a dict giving a vdwradii for each
         atom type which are used in guessing bonds.
-    is_anchor: bool, default ``True``
-        When unpickling instances of
-        :class:`MDAnalysis.core.groups.AtomGroup` existing Universes are
-        searched for one where to anchor those atoms. Set to ``False`` to
-        prevent this Universe from being considered.
-    anchor_name: str, ``None``, default ``None``
-        Setting to other than ``None`` will cause
-        :class:`MDAnalysis.core.groups.AtomGroup` instances pickled from the
-        Universe to only unpickle if a compatible Universe with matching
-        *anchor_name* is found. Even if *anchor_name* is set *is_anchor* will
-        still be honored when unpickling.
     transformations: function or list, ``None``, default ``None``
         Provide a list of transformations that you wish to apply to the
         trajectory upon reading. Transformations can be found in
@@ -302,47 +252,75 @@ class Universe(object):
         :mod:`ChainReader<MDAnalysis.coordinates.chain>`, which contains the
         functionality to treat independent trajectory files as a single virtual
         trajectory.
+    **kwargs: extra arguments are passed to the topology parser.
 
     Attributes
     ----------
-    trajectory
-        currently loaded trajectory reader;
-    dimensions
-        current system dimensions (simulation unit cell, if set in the
-        trajectory)
-    atoms, residues, segments
-        master Groups for each topology level
-    bonds, angles, dihedrals
-        master ConnectivityGroups for each connectivity type
+    trajectory : base.ReaderBase or base.SingleFrameReaderBase
+        currently loaded trajectory reader; readers are described in
+        :ref:`Coordinates`
+    dimensions : numpy.ndarray
+        system dimensions (simulation unit cell, if set in the
+        trajectory) at the *current time step*
+        (see :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`).
+        The unit cell can be set for the current time step (but the change is
+        not permanent unless written to a file).
+    atoms : AtomGroup
+        all particles (:class:`~MDAnalysis.core.groups.Atom`) in the system,
+        as read from the `topology` file
+    residues : ResidueGroup
+        all residues (:class:`~MDAnalysis.core.groups.Residue`) in the system
+    segments : SegmentGroup
+        all segments (:class:`~MDAnalysis.core.groups.Segment`) in the system
+    bonds : topologyattrs.Bonds
+        all bonds (if defined in the `topology`) as provided by
+        :attr:`Universe.atoms.bonds`
+    angles : topologyattrs.Angles
+        all angles (if defined in the `topology`), same as
+        :attr:`Universe.atoms.angles`
+    dihedrals : topologyattrs.Dihedrals
+        all dihedral angles (if defined in the `topology`), same as
+        :attr:`Universe.atoms.dihedrals`
+    impropers : topologyattrs.Impropers
+        all improper dihedral angles (if defined in the `topology`), same as
+        :attr:`Universe.atoms.impropers`
+
+    Examples
+    --------
+    Examples for setting up a :class:`Universe`::
+
+       u = Universe(topology, trajectory)          # read system from file(s)
+       u = Universe(pdbfile)                       # read atoms and coordinates from PDB or GRO
+       u = Universe(topology, [traj1, traj2, ...]) # read from a list of trajectories
+       u = Universe(topology, traj1, traj2, ...)   # read from multiple trajectories
+
+    Load new data into a universe (replaces old trajectory and does *not* append)::
+
+       u.load_new(trajectory)                      # read from a new trajectory file
+
+    Selecting atoms with :meth:`~Universe.select_atoms` ::
+
+       ag = u.select_atoms(...)
+
+    returns an :class:`~MDAnalysis.core.groups.AtomGroup`.
+
 
     .. versionchanged:: 1.0.0
         Universe() now raises an error. Use Universe(None) or :func:`Universe.empty()` instead.
         Removed instant selectors.
+
+    .. versionchanged:: 2.0.0
+        Universe now can be (un)pickled.
+        ``topology`` and ``trajectory`` are reserved upon unpickle.
+
     """
-# Py3 TODO
-#    def __init__(self, topology=None, *coordinates, all_coordinates=False,
-#                 format=None, topology_format=None, transformations=None,
-#                 guess_bonds=False, vdwradii=None, anchor_name=None,
-#                 is_anchor=True, in_memory=False, in_memory_step=1,
-#                 **kwargs):
-    def __init__(self, *args, **kwargs):
-        topology = args[0] if args else None
-        coordinates = args[1:]
-        all_coordinates = kwargs.pop('all_coordinates', False)
-        format = kwargs.pop('format', None)
-        topology_format = kwargs.pop('topology_format', None)
-        transformations = kwargs.pop('transformations', None)
-        guess_bonds = kwargs.pop('guess_bonds', False)
-        vdwradii = kwargs.pop('vdwradii', None)
-        anchor_name = kwargs.pop('anchor_name', None)
-        is_anchor = kwargs.pop('is_anchor', True)
-        in_memory = kwargs.pop('in_memory', False)
-        in_memory_step = kwargs.pop('in_memory_step', 1)
+    def __init__(self, topology=None, *coordinates, all_coordinates=False,
+                 format=None, topology_format=None, transformations=None,
+                 guess_bonds=False, vdwradii=None, in_memory=False,
+                 in_memory_step=1, **kwargs):
 
         self._trajectory = None  # managed attribute holding Reader
-        self._cache = {}
-        self._anchor_name = anchor_name
-        self.is_anchor = is_anchor
+        self._cache = {'_valid': {}}
         self.atoms = None
         self.residues = None
         self.segments = None
@@ -352,8 +330,6 @@ class Universe(object):
             'transformations': transformations,
             'guess_bonds': guess_bonds,
             'vdwradii': vdwradii,
-            'anchor_name': anchor_name,
-            'is_anchor': is_anchor,
             'in_memory': in_memory,
             'in_memory_step': in_memory_step,
             'format': format,
@@ -412,7 +388,7 @@ class Universe(object):
         Useful for building a Universe without requiring existing files,
         for example for system building.
 
-        If `trajectory` is set to True, a
+        If `trajectory` is set to ``True``, a
         :class:`MDAnalysis.coordinates.memory.MemoryReader` will be
         attached to the Universe.
 
@@ -431,33 +407,39 @@ class Universe(object):
         residue_segindex: array like, optional
           mapping of residues to segments
         trajectory: bool, optional
-          if True, attaches a :class:`MDAnalysis.coordinates.memory.MemoryReader`
-          allowing coordinates to be set and written.  Default is False
+          if ``True``, attaches a
+          :class:`MDAnalysis.coordinates.memory.MemoryReader` allowing
+          coordinates to be set and written.
         velocities: bool, optional
-          include velocities in the :class:`MDAnalysis.coordinates.memory.MemoryReader`
+          include velocities in the
+          :class:`MDAnalysis.coordinates.memory.MemoryReader`
         forces: bool, optional
-          include forces in the :class:`MDAnalysis.coordinates.memory.MemoryReader`
+          include forces in the
+          :class:`MDAnalysis.coordinates.memory.MemoryReader`
 
         Returns
         -------
-        MDAnalysis.Universe object
+        Universe
+          :class:`~MDAnalysis.core.universe.Universe` instance with dummy
+          values for atoms and undefined coordinates/velocities/forces
 
         Examples
         --------
         For example to create a new Universe with 6 atoms in 2 residues, with
-        positions for the atoms and a mass attribute:
+        positions for the atoms and a mass attribute::
 
-        >>> u = mda.Universe.empty(6, 2,
-                                   atom_resindex=np.array([0, 0, 0, 1, 1, 1]),
-                                   trajectory=True,
+          u = mda.Universe.empty(6, 2,
+                                 atom_resindex=np.array([0, 0, 0, 1, 1, 1]),
+                                 trajectory=True,
                 )
-        >>> u.add_TopologyAttr('masses')
+          u.add_TopologyAttr('masses')
 
         .. versionadded:: 0.17.0
         .. versionchanged:: 0.19.0
            The attached Reader when trajectory=True is now a MemoryReader
         .. versionchanged:: 1.0.0
            Universes can now be created with 0 atoms
+
         """
         if not n_atoms:
             n_residues = 0
@@ -484,14 +466,13 @@ class Universe(object):
 
         if trajectory:
             coords = np.zeros((1, n_atoms, 3), dtype=np.float32)
-            dims = np.zeros(6, dtype=np.float32)
             vels = np.zeros_like(coords) if velocities else None
             forces = np.zeros_like(coords) if forces else None
 
             # grab and attach a MemoryReader
             u.trajectory = get_reader_for(coords)(
                 coords, order='fac', n_atoms=n_atoms,
-                dimensions=dims, velocities=vels, forces=forces)
+                velocities=vels, forces=forces)
 
         return u
 
@@ -543,8 +524,9 @@ class Universe(object):
 
         Raises
         ------
-        TypeError if trajectory format can not be
-                  determined or no appropriate trajectory reader found
+        TypeError
+             if trajectory format can not be determined or no appropriate
+             trajectory reader found
 
 
         .. versionchanged:: 0.8
@@ -559,6 +541,7 @@ class Universe(object):
         .. versionchanged:: 0.17.0
            Now returns a :class:`Universe` instead of the tuple of file/array
            and detected file type.
+
         """
         # filename==None happens when only a topology is provided
         if filename is None:
@@ -676,63 +659,36 @@ class Universe(object):
 
     @property
     def bonds(self):
-        """Bonds between atoms"""
+        """Bonds between atoms.
+
+        :meta private:
+        """
         return self.atoms.bonds
 
     @property
     def angles(self):
-        """Angles between atoms"""
+        """Angles between atoms.
+
+        :meta private:
+        """
         return self.atoms.angles
 
     @property
     def dihedrals(self):
-        """Dihedral angles between atoms"""
+        """Dihedral angles between atoms.
+
+        :meta private:
+        """
         return self.atoms.dihedrals
 
     @property
     def impropers(self):
-        """Improper dihedral angles between atoms"""
+        """Improper dihedral angles between atoms.
+
+        :meta private:
+        """
         return self.atoms.impropers
 
-    @property
-    def anchor_name(self):
-        # hash used for anchoring.
-        # Try and use anchor_name, else use (and store) uuid
-        if self._anchor_name is not None:
-            return self._anchor_name
-        else:
-            try:
-                return self._anchor_uuid
-            except AttributeError:
-                # store this so we can later recall it if needed
-                self._anchor_uuid = uuid.uuid4()
-                return self._anchor_uuid
-
-    @anchor_name.setter
-    def anchor_name(self, name):
-        self.remove_anchor()  # clear any old anchor
-        self._anchor_name = str(name) if not name is None else name
-        self.make_anchor()  # add anchor again
-
-    @property
-    def is_anchor(self):
-        """Is this Universe an anchoring for unpickling AtomGroups"""
-        return self.anchor_name in _ANCHOR_UNIVERSES
-
-    @is_anchor.setter
-    def is_anchor(self, new):
-        if new:
-            self.make_anchor()
-        else:
-            self.remove_anchor()
-
-    def remove_anchor(self):
-        """Remove this Universe from the possible anchor list for unpickling"""
-        _ANCHOR_UNIVERSES.pop(self.anchor_name, None)
-
-    def make_anchor(self):
-        _ANCHOR_UNIVERSES[self.anchor_name] = self
-
     def __repr__(self):
         # return "<Universe with {n_atoms} atoms{bonds}>".format(
         #    n_atoms=len(self.atoms),
@@ -741,16 +697,29 @@ class Universe(object):
         return "<Universe with {n_atoms} atoms>".format(
             n_atoms=len(self.atoms))
 
-    def __getstate__(self):
-        raise NotImplementedError
+    @classmethod
+    def _unpickle_U(cls, top, traj):
+        """Special method used by __reduce__ to deserialise a Universe"""
+        #  top is a Topology obj at this point, but Universe can handle that.
+        u = cls(top)
+        u.trajectory = traj
 
-    def __setstate__(self, state):
-        raise NotImplementedError
+        return u
+
+    def __reduce__(self):
+        #  __setstate__/__getstate__ will raise an error when Universe has a
+        #  transformation (that has AtomGroup inside). Use __reduce__ instead.
+        #  Universe's two "legs" of top and traj both serialise themselves.
+        return (self._unpickle_U, (self._topology,
+                                   self._trajectory))
 
     # Properties
     @property
     def dimensions(self):
-        """Current dimensions of the unitcell"""
+        """Current dimensions of the unitcell.
+
+        :meta private:
+        """
         return self.coord.dimensions
 
     @dimensions.setter
@@ -789,7 +758,10 @@ class Universe(object):
 
     @property
     def trajectory(self):
-        """Reference to trajectory reader object containing trajectory data."""
+        """Reference to trajectory reader object containing trajectory data.
+
+        :meta private:
+        """
         if self._trajectory is not None:
             return self._trajectory
         else:
@@ -828,18 +800,30 @@ class Universe(object):
            Can now also add TopologyAttrs with a string of the name of the
            attribute to add (eg 'charges'), can also supply initial values
            using values keyword.
+
+        .. versionchanged:: 1.1.0
+            Now warns when adding bfactors to a Universe with
+            existing tempfactors, or adding tempfactors to a
+            Universe with existing bfactors.
+            In version 2.0, MDAnalysis will stop treating
+            tempfactors and bfactors as separate attributes. Instead,
+            they will be aliases of the same attribute.
         """
-        if isinstance(topologyattr, six.string_types):
+        if isinstance(topologyattr, str):
+            if topologyattr in ("bfactor", "bfactors"):
+                warnings.warn(BFACTOR_WARNING, DeprecationWarning)
             try:
                 tcls = _TOPOLOGY_ATTRS[topologyattr]
             except KeyError:
-                six.raise_from(ValueError(
+                errmsg = (
                     "Unrecognised topology attribute name: '{}'."
                     "  Possible values: '{}'\n"
-                    "To raise an issue go to: http://issues.mdanalysis.org"
+                    "To raise an issue go to: "
+                    "https://github.com/MDAnalysis/mdanalysis/issues"
                     "".format(
-                        topologyattr, ', '.join(sorted(_TOPOLOGY_ATTRS.keys())))),
-                    None)
+                        topologyattr, ', '.join(
+                            sorted(_TOPOLOGY_ATTRS.keys()))))
+                raise ValueError(errmsg) from None
             else:
                 topologyattr = tcls.from_blank(
                     n_atoms=self._topology.n_atoms,
@@ -849,6 +833,62 @@ class Universe(object):
         self._topology.add_TopologyAttr(topologyattr)
         self._process_attr(topologyattr)
 
+    def del_TopologyAttr(self, topologyattr):
+        """Remove a topology attribute from the Universe
+
+        Removing a TopologyAttribute from the Universe makes it unavailable to
+        all AtomGroups etc throughout the Universe.
+
+        Parameters
+        ----------
+        topologyattr: TopologyAttr or string
+          Either a MDAnalysis TopologyAttr object or the name of a possible
+          topology attribute.
+
+        Example
+        -------
+        For example to remove bfactors to a Universe:
+
+        >>> u.del_TopologyAttr('bfactors')
+        >>> hasattr(u.atoms[:3], 'bfactors')
+        False
+
+
+        .. versionadded:: 2.0.0
+        """
+
+        if not isinstance(topologyattr, str):
+            try:
+                topologyattr = topologyattr.attrname
+            except AttributeError:
+                # either TopologyGroup or not valid
+                try:
+                    # this may not end well
+                    # e.g. matrix -> matrices
+                    topologyattr = topologyattr.btype + "s"
+                except AttributeError:
+                    raise ValueError("Topology attribute must be str or "
+                                     "TopologyAttr object or class. "
+                                     f"Given: {type(topologyattr)}") from None
+
+        try:
+            topologyattr = _TOPOLOGY_ATTRS[topologyattr].attrname
+        except KeyError:
+            attrs = ', '.join(sorted(_TOPOLOGY_ATTRS))
+            errmsg = (f"Unrecognised topology attribute: '{topologyattr}'."
+                      f"  Possible values: '{attrs}'\n"
+                      "To raise an issue go to: "
+                      "https://github.com/MDAnalysis/mdanalysis/issues")
+            raise ValueError(errmsg) from None
+
+        try:
+            topattr = getattr(self._topology, topologyattr)
+        except AttributeError:
+            raise ValueError(f"Topology attribute {topologyattr} "
+                             "not in Universe.") from None
+        self._topology.del_TopologyAttr(topattr)
+        self._unprocess_attr(topattr)
+
     def _process_attr(self, attr):
         """Squeeze a topologyattr for its information
 
@@ -883,6 +923,24 @@ class Universe(object):
         for funcname, meth in attr.transplants['Universe']:
             setattr(self.__class__, funcname, meth)
 
+    def _unprocess_attr(self, attr):
+        """
+        Undo all the stuff in _process_attr.
+
+        If the topology attribute is not present, nothing happens
+        (silent fail).
+        """
+        for cls in attr.target_classes:
+            self._class_bases[cls]._del_prop(attr)
+
+        # Universe transplants
+        for funcname, _ in attr.transplants.pop("Universe", []):
+            delattr(self.__class__, funcname)
+        # Group transplants
+        for cls, transplants in attr.transplants.items():
+            for funcname, _ in transplants:
+                delattr(self._class_bases[cls], funcname)
+
     def add_Residue(self, segment=None, **attrs):
         """Add a new Residue to this Universe
 
@@ -1065,7 +1123,10 @@ class Universe(object):
         """
         self._add_topology_objects('bonds', values, types=types,
                                  guessed=guessed, order=order)
+        # Invalidate bond-related caches
         self._cache.pop('fragments', None)
+        self._cache['_valid'].pop('fragments', None)
+        self._cache['_valid'].pop('fragindices', None)
 
     def add_angles(self, values, types=None, guessed=False):
         """Add new Angles to this Universe.
@@ -1202,7 +1263,10 @@ class Universe(object):
         .. versionadded:: 1.0.0
         """
         self._delete_topology_objects('bonds', values)
+        # Invalidate bond-related caches
         self._cache.pop('fragments', None)
+        self._cache['_valid'].pop('fragments', None)
+        self._cache['_valid'].pop('fragindices', None)
 
     def delete_angles(self, values):
         """Delete Angles from this Universe.
@@ -1290,31 +1354,87 @@ class Universe(object):
 
         return fragdict
 
+    @classmethod
+    def from_smiles(cls, smiles, sanitize=True, addHs=True,
+                    generate_coordinates=True, numConfs=1,
+                    rdkit_kwargs={}, **kwargs):
+        """Create a Universe from a SMILES string with rdkit
 
-# TODO: what is the point of this function???
-def as_Universe(*args, **kwargs):
-    """Return a universe from the input arguments.
+        Parameters
+        ----------
+        smiles : str
+            SMILES string
+        sanitize : bool (optional, default True)
+            Toggle the sanitization of the molecule
+        addHs : bool (optional, default True)
+            Add all necessary hydrogens to the molecule
+        generate_coordinates : bool (optional, default True)
+            Generate 3D coordinates using RDKit's
+            :func:`AllChem.EmbedMultipleConfs` function. Requires adding
+            hydrogens with the `addHs` parameter
+        numConfs : int (optional, default 1)
+            Number of frames to generate coordinates for. Ignored if
+            ``generate_coordinates=False``
+        rdkit_kwargs : dict (optional)
+            Other arguments passed to the RDKit :func:`EmbedMultipleConfs`
+            function
+        kwargs : dict
+            Parameters passed on Universe creation
 
-    1. If the first argument is a universe, just return it::
+        Returns
+        -------
+        universe : Universe
+            contains atom names and topology information (bonds) derived from
+            the input SMILES string; coordinates are included if
+            `generate_coordinates` was set to ``True``
 
-         as_Universe(universe) --> universe
 
-    2. Otherwise try to build a universe from the first or the first
-       and second argument::
+        Examples
+        --------
+        To create a Universe with 10 conformers of ethanol:
 
-         as_Universe(PDB, **kwargs) --> Universe(PDB, **kwargs)
-         as_Universe(PSF, DCD, **kwargs) --> Universe(PSF, DCD, **kwargs)
-         as_Universe(*args, **kwargs) --> Universe(*args, **kwargs)
+        >>> u = mda.Universe.from_smiles('CCO', numConfs=10)
+        >>> u
+        <Universe with 9 atoms>
+        >>> u.trajectory
+        <RDKitReader with 10 frames of 9 atoms>
 
-    Returns
-    -------
-    :class:`~MDAnalysis.core.groups.Universe`
-    """
-    if len(args) == 0:
-        raise TypeError("as_Universe() takes at least one argument (%d given)" % len(args))
-    elif len(args) == 1 and isinstance(args[0], Universe):
-        return args[0]
-    return Universe(*args, **kwargs)
+        To use a different conformer generation algorithm, like ETKDGv3:
+
+        >>> u = mda.Universe.from_smiles('CCO', rdkit_kwargs=dict(
+                                         params=AllChem.ETKDGv3()))
+        >>> u.trajectory
+        <RDKitReader with 1 frames of 9 atoms>
+
+
+        .. versionadded:: 2.0.0
+
+        """
+        try:
+            from rdkit import Chem
+            from rdkit.Chem import AllChem
+        except ImportError as e:
+            raise ImportError(
+                "Creating a Universe from a SMILES string requires RDKit but "
+                "it does not appear to be installed") from e
+
+        mol = Chem.MolFromSmiles(smiles, sanitize=sanitize)
+        if mol is None:
+            raise SyntaxError('Error while parsing SMILES {0}'.format(smiles))
+        if addHs:
+            mol = Chem.AddHs(mol)
+        if generate_coordinates:
+            if not addHs:
+                raise ValueError("Generating coordinates requires adding "
+                "hydrogens with `addHs=True`")
+
+            numConfs = rdkit_kwargs.pop("numConfs", numConfs)
+            if not (type(numConfs) is int and numConfs > 0):
+                raise SyntaxError("numConfs must be a non-zero positive "
+                "integer instead of {0}".format(numConfs))
+            AllChem.EmbedMultipleConfs(mol, numConfs, **rdkit_kwargs)
+
+        return cls(mol, **kwargs)
 
 
 def Merge(*args):
